#pragma once
#define WIN32_LEAN_AND_MEAN //cut unused stuff out of windows
#define KEYDOWN(name, key) (name[key] & 0x80)
#include <windows.h> //includes for windows, directx, directinput, and classes
#include "dxhandlib.h"
#include <list>
#include <time.h>
#include <math.h>
#include "Ships.h"
#include "Mesh.h"
#include "keyboard.h"
#include "Font.h"
#include "Sprite.h"
#include "Texture.h"

using namespace std;

class Game
{
public:
	//Delcare functions (look at the definitions for more information
	void FatalError(const char *error_msg);
	static LRESULT CALLBACK default_window_proc(HWND hwnd,UINT msg,WPARAM wparam,LPARAM lparam);
	void init_window(void);
	void kill_window(void);
	void init_d3d(void);
	void kill_d3d(void);
	void init_scene(void);
	void kill_scene(void);
	void message_pump(void);
	D3DFORMAT find_16bit_mode(void);
	void render(void);
	void gameLogic(void);
	void run(void);
	void keyboard(void);
	void intro(void);
	void reset(void);
	void gameContinue(void);

	void InitGameTime();
	float GetGameTime();
	void UpdateFPS();
	Game();
	~Game();

private:
	int contNum; //temp variable for continuing countdown
	
	typedef enum //this enum tells us what the state of the game is, use to determine
	{			//what we are drawing during that frame
		INTRO, PLAYING, OVER
	} game_state;

	float  timeAtGameStart; //variables for frames per second calculations
	UINT64 ticksPerSecond;  //including time since lastupdate, how often to update the frames
	float lastUpdate;       //and the actual fps
	float fpsUpdateInterval;
	unsigned int  numFrames;
	float fps;

	HWND app_main_window;// global handle for main window

	// Main D3D interface, first object you create and last you release
	LPDIRECT3D8 app_D3D;

	struct Bullet //struct for our bullet information, used for player and enemy
	{
		D3DXVECTOR3 pos; //the 3d position of the bullet
		boundingSphere bSphere; //the bounding sphere around the bullet
	};

	Keyboard kInput; //create our directx objects
	Font fFont;		//these are actually created classes that use directx objects within
	Sprite mSprite; //them
	Texture introTexture;
	Texture playTexture;
	Texture livesTexture;
	game_state gState; //create a new gamestate

	Mesh mShip; //create our meshes from models that were exported to .x format
	Mesh mEnemy;
	Mesh mBullet;

	Ships player; //creates the player object, information to be filled in later
	float pRad;   //a radius value for the player

	list<Bullet> bullets; //player bullet list, need the iterator for movement through the
	list<Bullet>::iterator bIter; //list and temp for deletion and continuing with the
	list<Bullet>::iterator tempIter; //movement
	float bRad;
	int canShoot; //used to determine whether or not the player is able to shoot

	list<Ships> enemies; //the enemy list, iter and tempiter are same idea as bullet
	list<Ships>::iterator eIter;
	list<Ships>::iterator eTempIter;
	float eRad;

	list<Bullet> eBullets; //enemy bullets list, again with the iterators
	list<Bullet>::iterator eBIter;
	list<Bullet>::iterator eBtempIter;

	// global handle to our instance which is occasionally needed by Windows API
	HINSTANCE app_instance;

	D3DLIGHT8 light; //create a light
	int score; //self explanatory, the score the player has
};
