//the mesh function loads a mesh, optimizes it, gets the bounding sphere, and is able to draw it also

#include "Mesh.h"

Mesh::Mesh()
{
}

//the draw function as it is has much code that needs to be deleted as it is not used. Or what needs to be done is 
//allowing of rotation. for now we simply allow translation using a position vector
void Mesh::draw(LPDIRECT3DDEVICE8 app_device, D3DXVECTOR3 pos)
{
	D3DXMATRIX matWorld;
	D3DXMATRIX rot_matrix;     //rotation matrix
	D3DXMATRIX rot_matrix1;     //rotation matrix
	D3DXMATRIX rot_matrix2;     //rotation matrix
	D3DXMATRIX trans_matrix;

	  D3DXMatrixRotationX(&rot_matrix1,0);
	  D3DXMatrixRotationY(&rot_matrix2,0);
	  D3DXMatrixMultiply(&rot_matrix, &rot_matrix1, &rot_matrix2);
      D3DXMatrixTranslation(&trans_matrix, pos.x, pos.y, pos.z);
      D3DXMatrixMultiply(&matWorld,&rot_matrix,&trans_matrix); 

      //set the world matrix
      app_device->SetTransform(D3DTS_WORLD,&matWorld );

	  //draw mesh
	  for(DWORD j = 0; j < NumMaterials; j++)
        {
            app_device->SetMaterial(&MeshMaterials[j]);
            app_device->SetTexture(0, MeshTextures[j]);
        
            mesh->DrawSubset(j);
		}
}

//when deconstructing make sure to release the meshmaterials, tetures, and the mesh itself
Mesh::~Mesh()
{
	if( MeshMaterials)
    delete MeshMaterials;

	if( MeshTextures)
	{
		for( DWORD i = 0; i < NumMaterials; i++ )
		{
		 if( MeshTextures[i] )
		        MeshTextures[i]->Release();
	   }
	delete MeshTextures;
	}
	
	if( mesh )
		mesh->Release();
}

//we often will need the radius for other classes, such as the ships and the bounding sphere
float Mesh::getRadius()
{
	return radius;
}

void Mesh::Load(LPDIRECT3DDEVICE8 app_device, LPSTR fileName)
{
	LPD3DXBUFFER MaterialBuffer = NULL; //for loading mesh/model

 // Load the mesh from the specified file.
	D3DXLoadMeshFromX(fileName, D3DXMESH_SYSTEMMEM, 
                               app_device, NULL,
                               &MaterialBuffer, &NumMaterials, 
                               &mesh);

	D3DXMATERIAL* d3dxMaterials = (D3DXMATERIAL*)MaterialBuffer->GetBufferPointer();

	MeshMaterials = new D3DMATERIAL8[NumMaterials]; //array for mesh materials
    MeshTextures  = new LPDIRECT3DTEXTURE8[NumMaterials]; //array for mesh textures

	for(DWORD j = 0; j <NumMaterials; j++)
    {
	    MeshMaterials[j] = d3dxMaterials[j].MatD3D; //copy the mesh materials
        MeshMaterials[j].Ambient = MeshMaterials[j].Diffuse; //set ambient color
        
		// Create the texture for the mesh
		if( FAILED( D3DXCreateTextureFromFile( app_device, 
                                       d3dxMaterials[j].pTextureFilename, 
                                       &MeshTextures[j] ) ) )
		MeshTextures[j] = NULL;
	}

	D3DXComputeNormals(mesh, NULL); //compute the normals so lighting looks right
	mesh->OptimizeInplace(D3DXMESHOPT_VERTEXCACHE, NULL, NULL, NULL, NULL); //we optimize the mesh where it is

	MaterialBuffer->Release();//release the materialbuffer and return

	BYTE *ptr;
	D3DXVECTOR3 t;
	mesh->LockVertexBuffer(0, &ptr);
	//we must get the bounding sphere of the mesh so we can use it with collision detection later on
	D3DXComputeBoundingSphere((D3DXVECTOR3 *) ptr, mesh->GetNumVertices(), mesh->GetFVF(), &t, &radius);
	radius=radius;
    mesh->UnlockVertexBuffer();
	return;
}
